@@ -1,4 +1,5 @@
 package Sub::Frequency;
+
 use strict;
 use warnings;
 
@@ -6,43 +7,108 @@ use Scalar::Util 'looks_like_number';
 use Carp 'croak';
 
 use parent 'Exporter';
+
 our @EXPORT = qw(
-        always normally usually sometimes maybe
-        rarely seldom never with_probability
+  always normally usually often sometimes maybe
+  rarely seldom never with_probability
 );
+
 our @EXPORT_OK = @EXPORT;
 
+our $VERSION = '0.03';
 
-our $VERSION = '0.02';
+my %probabilities = (
+    'Sub::Frequency::Always'    => 1.00,
+    'Sub::Frequency::Normally'  => 0.75,
+    'Sub::Frequency::Sometimes' => 0.50,
+    'Sub::Frequency::Rarely'    => 0.25,
+    'Sub::Frequency::Never'     => 0.00,
+);
 
+sub always(&;@) {
+    my ( $code, @rest ) = @_;
+    my $name = 'Sub::Frequency::Always';
 
-sub always (&) { $_[0]->() }
+    if (wantarray) {
+        return ( bless( $code, $name ), @rest );
+    }
+    else {
+        _exec( $code, $name, @rest );
+    }
+}
 
+sub normally(&;@) {
+    my ( $code, @rest ) = @_;
+    my $name = 'Sub::Frequency::Normally';
 
-sub normally (&) {  with_probability( 0.75, @_ ) }
-*usually = \&normally;
+    if (wantarray) {
+        return ( bless( $code, $name ), @rest );
+    }
+    else {
+        _exec( $code, $name, @rest );
+    }
+}
 
+sub sometimes(&;@) {
+    my ( $code, @rest ) = @_;
+    my $name = 'Sub::Frequency::Sometimes';
 
-sub sometimes (&) {  with_probability( 0.5, @_ ) }
-*maybe = \&sometimes;
+    if (wantarray) {
+        return ( bless( $code, $name ), @rest );
+    }
+    else {
+        _exec( $code, $name, @rest );
+    }
+}
 
+sub rarely(&;@) {
+    my ( $code, @rest ) = @_;
+    my $name = 'Sub::Frequency::Rarely';
 
-sub rarely (&) {  with_probability( 0.25, @_ ) }
-*seldom = \&rarely;
+    if (wantarray) {
+        return ( bless( $code, $name ), @rest );
+    }
+    else {
+        _exec( $code, $name, @rest );
+    }
+}
 
+sub never(&;@) {
+    my ( $code, @rest ) = @_;
+    my $name = 'Sub::Frequency::Never';
 
-sub never (&)  { return }
-
+    if (wantarray) {
+        return ( bless( $code, $name ), @rest );
+    }
+    else {
+        _exec( $code, $name, @rest );
+    }
+}
 
 sub with_probability ($;&) {
-    my ($probability, $code) = @_;
+    my ( $probability, $code ) = @_;
 
     $probability = _coerce($probability)
-        unless looks_like_number($probability);
+      unless looks_like_number($probability);
 
     $code->() if rand() <= $probability;
 }
 
+*often   = \&normally;
+*usually = \&normally;
+*maybe   = \&sometimes;
+*seldom  = \&rarely;
+
+sub _exec {
+    my ( $code, $name, @rest ) = @_;
+
+    $code->() and return if rand() < $probabilities{$name};
+
+    foreach $code (@rest) {
+        $code->() and last if rand() < $probabilities{ ref($code) };
+    }
+}
+
 sub _coerce {
     my $thing = shift;
 
@@ -57,6 +123,7 @@ sub _coerce {
 
 42;
 __END__
+
 =head1 NAME
 
 Sub::Frequency - Run code blocks according to a given probability
@@ -94,6 +161,32 @@ You can also specify your own probability for the code to run:
         ...
     };
 
+Since version 0.03 you can chain probabilities together:
+
+    normally {
+
+        # code here will run 75% of the time
+
+    } maybe {
+
+        # code here will run 50% of the remaining 25% of the time,
+        # ie 12.5% of the total time
+
+    } seldom {
+
+        # code here will run 25% of the remaining 12.5% of the time,
+        # ie 3.125% of the total time
+
+    } always {
+
+        # code here will run on the remaining time, ie 9,375% of the time
+
+    };
+
+Note an absence of some semicolons compared with the previous examples.
+
+The function C<with_probability> cannot be chained yet.
+
 
 =head1 DESCRIPTION
 
@@ -120,6 +213,8 @@ Takes a mandatory subroutine and executes it every time.
 
 =head2 normally
 
+=head2 often
+
 Takes a mandatory subroutine and executes it with a probability of 75%.
 
 
@@ -189,9 +284,13 @@ between 0 and 1, or a percentage string (such as '15%').
 
 =head1 AUTHORS
 
-Breno G. de Oliveira, C<< <garu at cpan.org> >>
+Breno G. de Oliveira (garu), C<< <garu at cpan.org> >>
 
-Tiago Peczenyj
+Tiago Peczenyj (pac-man)
+
+=head1 CONTRIBUTORS
+
+Wesley Dal`Col (blabos) C<< <blabos at cpan.org> >>
 
 
 =head1 BUGS
